// -*- C++ -*-
#ifndef RIVET_SubtractedJewelEvent_HH
#define RIVET_SubtractedJewelEvent_HH

#include "Rivet/Projection.hh"
#include "Rivet/Particle.hh"

namespace Rivet {

    struct My4Mom {
	    double pt;
	    double mdelta;
	    double phi;
	    double y;
    } ;

    struct MyPart {
	    double pt;
	    double mdelta;
	    double phi;
	    double y;
	    PdgId  id;
    } ;

    struct MyDist {
	    double dR;
	    size_t ipart;
	    size_t itherm;
    } ;
    
    inline bool MyDistComp (const MyDist& dist1, const MyDist& dist2) {
	    return dist1.dR < dist2.dR; 
    }

  class SubtractedJewelEvent : public Projection {
  public:

    SubtractedJewelEvent(double dRmax) :
      _dRmax(dRmax) {
      setName("SubtractedJewelEvent");
    }

    // Clone on the heap.
    DEFAULT_RIVET_PROJ_CLONE(SubtractedJewelEvent);

    /// Copy constructor
    SubtractedJewelEvent(const SubtractedJewelEvent& other) :
      _dRmax(other._dRmax) {
    }

    CmpState compare(const Projection& p) const override
    {
      const SubtractedJewelEvent* other = dynamic_cast<const SubtractedJewelEvent*>(&p);
      if (!other) return CmpState::NEQ;
      if (_dRmax != other->_dRmax) return CmpState::NEQ;
      return CmpState::EQ;
    }
    
    const Particles& subtractedEvent() const { return _subtrevent;}
    
  protected:

    void project(const Event& e) override;

    void clear();
    void translateParticles(const Event& e); 
    vector<MyDist> buildPairs();
    void doSubtraction(vector<MyDist> & dist);
    void collectResult();
    
    double _dRmax;
    Particles            _subtrevent;
    vector<MyPart>       _myparticles;
    vector<My4Mom>       _mythermals;

};


}

#endif
