// -*- C++ -*-
#include "Rivet/Projections/SubtractedJewelFinalState.hh"

namespace Rivet {


  SubtractedJewelFinalState::SubtractedJewelFinalState(const SubtractedJewelEvent& sev, const Cut& c) {
    setName("SubtractedJewelFinalState");
    declare(sev, "SubtrEv");
  }


  CmpState SubtractedJewelFinalState::compare(const Projection& p) const {
    const SubtractedJewelFinalState* other = dynamic_cast<const SubtractedJewelFinalState*>(&p);
    if (!other) return CmpState::NEQ;
    // First check if the subtracted events match
    const PCmp sevcmp = mkPCmp(*other, "SubtrEv");
    if (sevcmp != CmpState::EQ) return  CmpState::NEQ;
    // Then check the cuts
    const bool cutcmp = _cuts == other->_cuts;
    if (!cutcmp) return CmpState::NEQ;
    // Checks all passed: these FSes are equivalent
    return CmpState::EQ;
  }

  void SubtractedJewelFinalState::project(const Event& e) {
    _theParticles.clear();
    const Particles& allparticles = applyProjection<SubtractedJewelEvent>(e, "SubtrEv").subtractedEvent();
    for (const Particle& p : allparticles) {
      const bool passed = accept(p);
      if (passed) _theParticles.push_back(p);
    }
  }

  /// Decide if a particle is to be accepted or not.
  bool SubtractedJewelFinalState::accept(const Particle& p) const {
    assert(p.genParticle() == NULL);
    return _cuts->accept(p);
  }

}
